@@ -57,9 +57,6 @@ module Network
 
     -- ** Improving I\/O Performance over sockets
     {-$performance-}
-
-    -- ** @SIGPIPE@
-    {-$sigpipe-}
     ) where
 
 import Control.Monad (liftM)
@@ -83,7 +80,7 @@ import qualified Control.Exception as Exception
 data PortID =
           Service String                -- Service Name eg "ftp"
         | PortNumber PortNumber         -- User defined Port Number
-#if !defined(mingw32_HOST_OS) && !defined(cygwin32_HOST_OS) && !defined(_WIN32)
+#if !defined(mingw32_HOST_OS)
         | UnixSocket String             -- Unix family socket in file system
 #endif
         deriving (Show, Eq)
@@ -100,9 +97,9 @@ connectTo :: HostName           -- Hostname
 #if defined(IPV6_SOCKET_SUPPORT)
 -- IPv6 and IPv4.
 
-connectTo hostname (Service serv) = connect' hostname serv
+connectTo hostname (Service serv) = connect' "Network.connectTo" hostname serv
 
-connectTo hostname (PortNumber port) = connect' hostname (show port)
+connectTo hostname (PortNumber port) = connect' "Network.connectTo" hostname (show port)
 #else
 -- IPv4 only.
 
@@ -130,7 +127,7 @@ connectTo hostname (PortNumber port) = do
         )
 #endif
 
-#if !defined(mingw32_HOST_OS) && !defined(cygwin32_HOST_OS) && !defined(_WIN32)
+#if !defined(mingw32_HOST_OS)
 connectTo _ (UnixSocket path) = do
     bracketOnError
         (socket AF_UNIX Stream 0)
@@ -142,15 +139,15 @@ connectTo _ (UnixSocket path) = do
 #endif
 
 #if defined(IPV6_SOCKET_SUPPORT)
-connect' :: HostName -> ServiceName -> IO Handle
+connect' :: String -> HostName -> ServiceName -> IO Handle
 
-connect' host serv = do
+connect' caller host serv = do
     proto <- getProtocolNumber "tcp"
     let hints = defaultHints { addrFlags = [AI_ADDRCONFIG]
                              , addrProtocol = proto
                              , addrSocketType = Stream }
     addrs <- getAddrInfo (Just hints) (Just host) (Just serv)
-    firstSuccessful $ map tryToConnect addrs
+    firstSuccessful caller $ map tryToConnect addrs
   where
   tryToConnect addr =
     bracketOnError
@@ -198,7 +195,7 @@ listenOn (Service serv) = do
         (\sock -> do
             port    <- getServicePortNumber serv
             setSocketOption sock ReuseAddr 1
-            bindSocket sock (SockAddrInet port iNADDR_ANY)
+            bind sock (SockAddrInet port iNADDR_ANY)
             listen sock maxListenQueue
             return sock
         )
@@ -210,20 +207,20 @@ listenOn (PortNumber port) = do
         (sClose)
         (\sock -> do
             setSocketOption sock ReuseAddr 1
-            bindSocket sock (SockAddrInet port iNADDR_ANY)
+            bind sock (SockAddrInet port iNADDR_ANY)
             listen sock maxListenQueue
             return sock
         )
 #endif
 
-#if !defined(mingw32_HOST_OS) && !defined(cygwin32_HOST_OS) && !defined(_WIN32)
+#if !defined(mingw32_HOST_OS)
 listenOn (UnixSocket path) =
     bracketOnError
         (socket AF_UNIX Stream 0)
         (sClose)
         (\sock -> do
             setSocketOption sock ReuseAddr 1
-            bindSocket sock (SockAddrUnix path)
+            bind sock (SockAddrUnix path)
             listen sock maxListenQueue
             return sock
         )
@@ -250,7 +247,7 @@ listen' serv = do
         (sClose)
         (\sock -> do
             setSocketOption sock ReuseAddr 1
-            bindSocket sock (addrAddress addr)
+            bind sock (addrAddress addr)
             listen sock maxListenQueue
             return sock
         )
@@ -292,9 +289,16 @@ accept sock@(MkSocket _ AF_INET6 _ _ _) = do
          \_ -> case addr of
                  SockAddrInet  _   a   -> inet_ntoa a
                  SockAddrInet6 _ _ a _ -> return (show a)
-# if !defined(mingw32_HOST_OS) && !defined(cygwin32_HOST_OS) && !defined(_WIN32)
+#if defined(mingw32_HOST_OS)
+                 SockAddrUnix {}       -> ioError $ userError "Network.accept: peer socket address 'SockAddrUnix' not supported on this platform."
+#else
                  SockAddrUnix      a   -> return a
-# endif
+#endif
+#if defined(CAN_SOCKET_SUPPORT)
+                 SockAddrCan {}        -> ioError $ userError "Network.accept: peer socket address 'SockAddrCan' not supported."
+#else
+                 SockAddrCan {}        -> ioError $ userError "Network.accept: peer socket address 'SockAddrCan' not supported on this platform."
+#endif
  handle <- socketToHandle sock' ReadWriteMode
  let port = case addr of
               SockAddrInet  p _     -> p
@@ -302,20 +306,21 @@ accept sock@(MkSocket _ AF_INET6 _ _ _) = do
               _                     -> -1
  return (handle, peer, port)
 #endif
-#if !defined(mingw32_HOST_OS) && !defined(cygwin32_HOST_OS) && !defined(_WIN32)
+#if !defined(mingw32_HOST_OS)
 accept sock@(MkSocket _ AF_UNIX _ _ _) = do
  ~(sock', (SockAddrUnix path)) <- Socket.accept sock
  handle <- socketToHandle sock' ReadWriteMode
  return (handle, path, -1)
 #endif
 accept (MkSocket _ family _ _ _) =
-  error $ "Sorry, address family " ++ (show family) ++ " is not supported!"
+  ioError $ userError $ "Network.accept: address family '" ++
+    show family ++ "' not supported."
 
 
--- | Close the socket. All future operations on the socket object will fail.
---   The remote end will receive no more data (after queued data is flushed).
+-- | Close the socket. Sending data to or receiving data from closed socket
+-- may lead to undefined behaviour.
 sClose :: Socket -> IO ()
-sClose = close -- Explicit redefinition because Network.sClose is deperecated,
+sClose = close -- Explicit redefinition because Network.sClose is deprecated,
                -- hence the re-export would also be marked as such.
 
 -- -----------------------------------------------------------------------------
@@ -395,17 +400,19 @@ recvFrom host port = do
 socketPort :: Socket -> IO PortID
 socketPort s = do
     sockaddr <- getSocketName s
-    return (portID sockaddr)
-  where
-   portID sa =
-    case sa of
-     SockAddrInet port _      -> PortNumber port
+    case sockaddr of
+      SockAddrInet port _      -> return $ PortNumber port
 #if defined(IPV6_SOCKET_SUPPORT)
-     SockAddrInet6 port _ _ _ -> PortNumber port
+      SockAddrInet6 port _ _ _ -> return $ PortNumber port
+#else
+      SockAddrInet6 {}         -> ioError $ userError "Network.socketPort: socket address 'SockAddrInet6' not supported on this platform."
 #endif
-#if !defined(mingw32_HOST_OS) && !defined(cygwin32_HOST_OS) && !defined(_WIN32)
-     SockAddrUnix path        -> UnixSocket path
+#if defined(mingw32_HOST_OS)
+      SockAddrUnix {}          -> ioError $ userError "Network.socketPort: socket address 'SockAddrUnix' not supported on this platform."
+#else
+      SockAddrUnix path        -> return $ UnixSocket path
 #endif
+      SockAddrCan {}           -> ioError $ userError "Network.socketPort: socket address 'SockAddrCan' not supported."
 
 -- ---------------------------------------------------------------------------
 -- Utils
@@ -439,21 +446,6 @@ For really fast I\/O, it might be worth looking at the 'hGetBuf' and
 'hPutBuf' family of functions in "System.IO".
 -}
 
-{-$sigpipe
-
-On Unix, when writing to a socket and the reading end is
-closed by the remote client, the program is normally sent a
-@SIGPIPE@ signal by the operating system.  The
-default behaviour when a @SIGPIPE@ is received is
-to terminate the program silently, which can be somewhat confusing
-if you haven't encountered this before.  The solution is to
-specify that @SIGPIPE@ is to be ignored, using
-the POSIX library:
-
->  import Posix
->  main = do installHandler sigPIPE Ignore Nothing; ...
--}
-
 catchIO :: IO a -> (Exception.IOException -> IO a) -> IO a
 #if MIN_VERSION_base(4,0,0)
 catchIO = Exception.catch
@@ -471,8 +463,8 @@ tryIO m = catchIO (liftM Right m) (return . Left)
 -- The operations are run outside of the catchIO cleanup handler because
 -- catchIO masks asynchronous exceptions in the cleanup handler.
 -- In the case of complete failure, the last exception is actually thrown.
-firstSuccessful :: [IO a] -> IO a
-firstSuccessful = go Nothing
+firstSuccessful :: String -> [IO a] -> IO a
+firstSuccessful caller = go Nothing
   where
   -- Attempt the next operation, remember exception on failure
   go _ (p:ps) =
@@ -482,5 +474,5 @@ firstSuccessful = go Nothing
          Left  e -> go (Just e) ps
 
   -- All operations failed, throw error if one exists
-  go Nothing  [] = error "firstSuccessful: empty list"
+  go Nothing  [] = ioError $ userError $ caller ++ ": firstSuccessful: empty list"
   go (Just e) [] = Exception.throwIO e
