@@ -5,11 +5,14 @@ module Main where
 import Control.Concurrent (ThreadId, forkIO, myThreadId)
 import Control.Concurrent.MVar (newEmptyMVar, putMVar, takeMVar, readMVar)
 import qualified Control.Exception as E
-import Control.Monad (liftM, when)
+import Control.Monad
 import qualified Data.ByteString as S
 import qualified Data.ByteString.Char8 as C
+#if defined(HAVE_LINUX_CAN_H)
 import Data.Maybe (fromJust)
+#endif
 import Network.Socket hiding (recv, recvFrom, send, sendTo)
+import qualified Network.Socket (recv)
 import Network.Socket.ByteString
 
 --- To tests for AF_CAN on Linux, you need to bring up a virtual (or real can
@@ -28,7 +31,7 @@ import Network.BSD (ifNameToIndex)
 #endif
 import Test.Framework (Test, defaultMain, testGroup)
 import Test.Framework.Providers.HUnit (testCase)
-import Test.HUnit (Assertion, (@=?))
+import Test.HUnit (Assertion, (@=?), assertBool, assertFailure)
 
 ------------------------------------------------------------------------
 
@@ -137,7 +140,7 @@ testUserTimeout = do
       getSocketOption sock UserTimeout >>= (@=?) 1000
       setSocketOption sock UserTimeout 2000
       getSocketOption sock UserTimeout >>= (@=?) 2000
-      sClose sock
+      close sock
 
 {-
 testGetPeerCred:: Assertion
@@ -146,22 +149,22 @@ testGetPeerCred =
   where
     clientSetup = do
         sock <- socket AF_UNIX Stream defaultProtocol
-        connect sock $ SockAddrUnix addr 
+        connect sock $ SockAddrUnix addr
         return sock
 
     serverSetup = do
         sock <- socket AF_UNIX Stream defaultProtocol
-        bindSocket sock $ SockAddrUnix addr 
+        bind sock $ SockAddrUnix addr 
         listen sock 1
         return sock
 
     server sock = do
         (clientSock, _) <- accept sock
-        serverAct clientSock
-        sClose clientSock
+        _ <- serverAct clientSock
+        close clientSock
 
     addr = "/tmp/testAddr1"
-    clientAct sock = withSocketsDo $ do  
+    clientAct sock = withSocketsDo $ do
                      sendAll sock testMsg
                      (pid,uid,gid) <- getPeerCred sock
                      putStrLn $ unwords ["pid=",show pid,"uid=",show uid, "gid=", show gid]
@@ -171,27 +174,27 @@ testGetPeerCred =
 
 
 testGetPeerEid :: Assertion
-testGetPeerEid =  
+testGetPeerEid =
     test clientSetup clientAct serverSetup server
   where
     clientSetup = do
         sock <- socket AF_UNIX Stream defaultProtocol
-        connect sock $ SockAddrUnix addr 
+        connect sock $ SockAddrUnix addr
         return sock
 
     serverSetup = do
         sock <- socket AF_UNIX Stream defaultProtocol
-        bindSocket sock $ SockAddrUnix addr 
+        bind sock $ SockAddrUnix addr 
         listen sock 1
         return sock
 
     server sock = do
         (clientSock, _) <- accept sock
-        serverAct clientSock
-        sClose clientSock
+        _ <- serverAct clientSock
+        close clientSock
 
     addr = "/tmp/testAddr2"
-    clientAct sock = withSocketsDo $ do  
+    clientAct sock = withSocketsDo $ do
                      sendAll sock testMsg
                      (uid,gid) <- getPeerEid sock
                      putStrLn $ unwords ["uid=",show uid, "gid=", show gid]
@@ -223,10 +226,71 @@ canTest ifname clientAct serverAct = do
       -- bind the socket to the interface
       bind sock (SockAddrCan $ fromIntegral $ ifIndex)
       return sock
-    
+
     serverSetup = clientSetup
 #endif
 
+-- The String version of 'recv' should throw an exception when the remote end
+-- has closed the connection, the ByteString version should return an empty
+-- ByteString.
+testStringEol :: Assertion
+testStringEol = tcpTest client close
+  where client s = do
+          res <- E.try $ Network.Socket.recv s 4096
+          case res of
+            Left (_ :: IOError) -> return ()
+            Right _ -> assertFailure
+              "String recv didn't throw an exception on a closed socket"
+
+testByteStringEol :: Assertion
+testByteStringEol = tcpTest client close
+  where client s = do
+          res :: Either IOError C.ByteString <- E.try $ recv s 4096
+          res @=? Right S.empty
+
+------------------------------------------------------------------------
+-- Conversions of IP addresses
+
+testHtonlNtohl :: Assertion
+testHtonlNtohl = do
+    let addrl = 0xCafeBabe
+    addrl @=? (htonl . ntohl) addrl
+    addrl @=? (ntohl . htonl) addrl
+    assertBool "BE or LE byte order" $
+        ntohl addrl `elem` [0xCafeBabe, 0xbebafeca]
+
+testHostAddressToTuple :: Assertion
+testHostAddressToTuple = do
+    -- Look up a numeric IPv4 host
+    let hints = defaultHints { addrFlags = [AI_NUMERICHOST, AI_ADDRCONFIG] }
+    (AddrInfo{addrAddress = (SockAddrInet _ hostAddr)} : _) <-
+        getAddrInfo (Just hints) (Just "127.128.129.130") Nothing
+    -- and check that the decoded address matches the expected representation
+    (0x7f, 0x80, 0x81, 0x82) @=? hostAddressToTuple hostAddr
+
+testHostAddressToTupleInv :: Assertion
+testHostAddressToTupleInv = do
+    let addr = (0x7f, 0x80, 0x81, 0x82)
+    addr @=? (hostAddressToTuple . tupleToHostAddress) addr
+
+#if defined(IPV6_SOCKET_SUPPORT)
+testHostAddress6ToTuple :: Assertion
+testHostAddress6ToTuple = do
+    -- Look up a numeric IPv6 host
+    let hints = defaultHints { addrFlags = [AI_NUMERICHOST, AI_ADDRCONFIG] }
+        host = "2001:0db8:85a3:0000:0000:8a2e:0370:7334"
+    (AddrInfo{addrAddress = (SockAddrInet6 _ _ hostAddr _)} : _) <-
+        getAddrInfo (Just hints) (Just host) Nothing
+    -- and check that the decoded address matches the expected representation
+    (0x2001, 0x0db8, 0x85a3, 0x0000, 0x0000, 0x8a2e, 0x0370, 0x7334)
+        @=? hostAddress6ToTuple hostAddr
+
+testHostAddress6ToTupleInv :: Assertion
+testHostAddress6ToTupleInv = do
+    let addr = (0x2001, 0x0db8, 0x85a3, 0x0000, 0x0000, 0x8a2e, 0x0370, 0x7334)
+    addr @=? (hostAddress6ToTuple . tupleToHostAddress6) addr
+#endif
+
 ------------------------------------------------------------------------
 -- Other
 
@@ -250,9 +314,20 @@ basicTests = testGroup "Basic socket operations"
     , testCase "testUserTimeout" testUserTimeout
 --    , testCase "testGetPeerCred" testGetPeerCred
 --    , testCase "testGetPeerEid" testGetPeerEid
+    , testCase "testStringEol" testStringEol
+    , testCase "testByteStringEol" testByteStringEol
 #if defined(HAVE_LINUX_CAN_H)
-    , testCase "testCanSend" testCanSend  
+    , testCase "testCanSend" testCanSend
 #endif
+      -- conversions of IP addresses
+    , testCase "testHtonlNtohl" testHtonlNtohl
+    , testCase "testHostAddressToTuple" testHostAddressToTuple
+    , testCase "testHostAddressToTupleInv" testHostAddressToTupleInv
+#if defined(IPV6_SOCKET_SUPPORT)
+    , testCase "testHostAddress6ToTuple" testHostAddress6ToTuple
+    , testCase "testHostAddress6ToTupleInv" testHostAddress6ToTupleInv
+#endif
+      -- other
     ]
 
 tests :: [Test]
@@ -269,7 +344,7 @@ getPeerPort sock = do
     case sockAddr of
         (SockAddrInet port _) -> return port
         (SockAddrInet6 port _ _ _) -> return port
-        _ -> error "getPeerPort: only works with IP sockets"
+        _ -> ioError $ userError "getPeerPort: only works with IP sockets"
 
 -- | Establish a connection between client and server and then run
 -- 'clientAct' and 'serverAct', in different threads.  Both actions
@@ -292,7 +367,7 @@ tcpTest clientAct serverAct = do
         sock <- socket AF_INET Stream defaultProtocol
         setSocketOption sock ReuseAddr 1
         addr <- inet_addr serverAddr
-        bindSocket sock (SockAddrInet aNY_PORT addr)
+        bind sock (SockAddrInet aNY_PORT addr)
         listen sock 1
         serverPort <- socketPort sock
         putMVar portVar serverPort
@@ -300,8 +375,8 @@ tcpTest clientAct serverAct = do
 
     server sock = do
         (clientSock, _) <- accept sock
-        serverAct clientSock
-        sClose clientSock
+        _ <- serverAct clientSock
+        close clientSock
 
 -- | Create an unconnected 'Socket' for sending UDP and receiving
 -- datagrams and then run 'clientAct' and 'serverAct'.
@@ -320,7 +395,7 @@ udpTest clientAct serverAct = do
         sock <- socket AF_INET Datagram defaultProtocol
         setSocketOption sock ReuseAddr 1
         addr <- inet_addr serverAddr
-        bindSocket sock (SockAddrInet aNY_PORT addr)
+        bind sock (SockAddrInet aNY_PORT addr)
         serverPort <- socketPort sock
         putMVar portVar serverPort
         return sock
@@ -332,13 +407,13 @@ test :: IO Socket -> (Socket -> IO b) -> IO Socket -> (Socket -> IO c) -> IO ()
 test clientSetup clientAct serverSetup serverAct = do
     tid <- myThreadId
     barrier <- newEmptyMVar
-    forkIO $ server barrier
+    _ <- forkIO $ server barrier
     client tid barrier
   where
     server barrier = do
-        E.bracket serverSetup sClose $ \sock -> do
+        E.bracket serverSetup close $ \sock -> do
             serverReady
-            serverAct sock
+            _ <- serverAct sock
             putMVar barrier ()
       where
         -- | Signal to the client that it can proceed.
@@ -347,8 +422,8 @@ test clientSetup clientAct serverSetup serverAct = do
     client tid barrier = do
         takeMVar barrier
         -- Transfer exceptions to the main thread.
-        bracketWithReraise tid clientSetup sClose $ \res -> do
-            clientAct res
+        bracketWithReraise tid clientSetup close $ \res -> do
+            _ <- clientAct res
             takeMVar barrier
 
 -- | Like 'bracket' but catches and reraises the exception in another
